from __future__ import absolute_import
from circuitsim.component import Component
from solver4 import Expr, Variable, Constraint

class Capacitor(Component):
	
	def __init__(self, capacitance, circuit=None):
		Component.__init__(self)
		
		self._capacitance = capacitance
		
		self._current = Variable()
		self._current.value = 0
		self._vars.add(self._current)
		
		self._charge = Variable()
		self._charge.derivative = self._current
		self._charge.value = 0
		self._vars.add(self._charge)
		
		self._C1 = Constraint(Expr.constant(1))
		self._C1.max_iterations = 1
		self._constraints.add(self._C1)
		
		self._update_names()
		
		self._pins[0] = None
		self._pins[1] = None
		
		if circuit:
			self.install(circuit)
	
	def current(self, pin):
		if pin==0:
			return self._current.value
		else:
			return -self._current.value
	
	def _get_current(self, pin):
		if pin==0:
			return Expr.variable(self._current)
		else:
			return Expr.product(Expr.variable(self._current), Expr.constant(-1))
	
	def _update_constraints(self):
		# Q = C*(V[0]-V[1])
		# ===> Q - C*V[0] + C*V[1] = 0
		self._C1.expr = Expr.sum(
			Expr.variable(self._charge),
			Expr.product(Expr.constant(-self._capacitance), Expr.variable(self._pins[0]._voltage)),
			Expr.product(Expr.constant(self._capacitance), Expr.variable(self._pins[1]._voltage))
			)
	
	def _update_names(self):
		self._current.name = "current of %s" % self
		self._charge.name = "charge of %s" % self
		self._C1.name = str(self)
	
	def get_capacitance(self):
		return self._capacitance
	
	def set_capacitance(self, c):
		self._capacitance = c
		self._update_constraints()
	
	capacitance = property(get_capacitance, set_capacitance)
	
	def __repr__(self):
		return "<%s (%f F) at %s>" % \
			(self.__class__.__name__, self._capacitance, self.name or hex(id(self)))
		