from __future__ import absolute_import
from circuitsim.wires import unwire

class Component(object):
	def __init__(self):
		self._pins = {}
		self._constraints = set()
		self._vars = set()
		self._circuit = None
		self._name = None
	
	def install(self, circuit):
		assert not self._circuit
		
		self._circuit = circuit
		self._circuit._components.add(self)
		
		for var in self._vars:
			self._circuit._sim.add_variable(var)
		for constraint in self._constraints:
			self._circuit._sim.add_constraint(constraint)
	
	def remove(self):
		assert self._circuit
		
		for pin in self._pins:
			if self._pins[pin]:
				unwire(self, pin)
		
		for var in self._vars:
			self._circuit._sim.remove_variable(var)
		for constraint in self._constraints:
			self._circuit._sim.remove_constraint(constraint)
		
		self._circuit._components.remove(self)
		self._circuit = None
	
	def _update_constraints_if_possible(self):
		if all(self._pins.values()):
			self._update_constraints()
	
	def _get_current(self, pin):
		# A positive return value indicates electrons flowing out of component
		raise NotImplementedError
	
	def _update_constraints(self):
		raise NotImplementedError
	
	def _update_names(self):
		pass
	
	def get_name(self):
		if self._name:
			return self._name
		else:
			return ""
	
	def set_name(self, name):
		self._name = name
		self._update_names()
	
	name = property(get_name, set_name)