from __future__ import absolute_import
from circuitsim.component import Component
from solver4 import Expr, Variable, Constraint

class Ground(Component):
	
	def __init__(self, circuit=None):
		Component.__init__(self)
		
		self._pins[0] = None
		
		self._current = Variable()
		self._current.value = 0
		self._vars.add(self._current)
		
		self._C1 = Constraint(Expr.constant(1))
		self._C1.max_iterations = 1
		self._constraints.add(self._C1)
		
		self._update_names()
		
		if circuit:
			self.install(circuit)
	
	def install(self,circuit):
		Component.install(self, circuit)
		self._circuit._update_ground_current_constraint()
	
	def remove(self):
		c = self._circuit
		Component.remove(self)
		c._update_ground_current_constraint()
	
	def current(self, pin):
		return self._current.value
	
	def _get_current(self, pin):
		return Expr.variable(self._current)
	
	def _update_constraints(self):
		# The constraint on current through ground is created and maintained by the circuit object. 
		# See circuit._update_ground_currrent_constraint.
		
		# V[0] = 0
		self._C1.expr = Expr.variable(self._pins[0]._voltage)
	
	def _update_names(self):
		self._current.name = "current of %s" % self
		self._C1.name = "zero-voltage constraint of %s" % self
	
	def __repr__(self):
		return "<%s %s>" % (self.__class__.__name__, self.name or hex(id(self)))