from __future__ import absolute_import
from solver4 import Expr, Variable, Constraint
from circuitsim.wires import unwire

class Node(object):
	def __init__(self, circuit):
		self._circuit = circuit
		self._circuit._nodes.add(self)

		self._voltage = Variable()
		self._voltage.value = 0
		self._voltage.name = "voltage of %s" % hex(id(self))
		self._circuit._sim.add_variable(self._voltage)
		
		self._current_constraint = Constraint(Expr.constant(0))
		self._current_constraint.max_iterations = 1
		self._current_constraint.name = "current constraint of %s" % hex(id(self))
		self._circuit._sim.add_constraint(self._current_constraint)
				
		self._wires = set()
	
	def delete(self):
		for (component,pin) in self._wires:
			unwire(component, pin)
		del self._wires
		
		self._circuit._sim.remove_variable(self._voltage)
		del self._voltage
		
		self._circuit._sim.remove_constraint(self._current_constraint)
		del self._current_constraint
		
		self._circuit._nodes.remove(self)
		self._circuit = None
	
	def voltage(self):
		return self._voltage.value
	
	def _update_current_constraint(self):
		currents = [component._get_current(pin) for (component,pin) in self._wires]
		assert all(isinstance(c,Expr) for c in currents)
		e = Expr.sum(*currents)
		self._current_constraint.expr = e