from __future__ import absolute_import
from circuitsim.component import Component
from solver4 import Expr, Variable, Constraint

class Resistor(Component):
	
	def __init__(self, resistance, circuit=None):
		Component.__init__(self)
		
		self._resistance = resistance
		
		self._current = Variable()
		self._current.value = 0.0
		self._vars.add(self._current)
		
		self._C1 = Constraint(Expr.constant(1))
		self._C1.max_iterations = 1
		self._constraints.add(self._C1)
		
		self._update_names()
		
		self._pins[0] = None
		self._pins[1] = None
		
		if circuit:
			self.install(circuit)
	
	def current(self, pin):
		if pin==0:
			return self._current.value
		else:
			return -self._current.value
	
	def _get_current(self, pin):
		if pin==0:
			return Expr.variable(self._current)
		else:
			return Expr.product(Expr.variable(self._current), Expr.constant(-1))
	
	def _update_constraints(self):
		# V[0] - V[1] - IR = 0
		self._C1.expr = Expr.sum(
			Expr.variable(self._pins[0]._voltage),
			Expr.product(Expr.constant(-1), Expr.variable(self._pins[1]._voltage)),
			Expr.product(Expr.constant(-self._resistance), Expr.variable(self._current))
			)
	
	def _update_names(self):
		self._current.name = "current of %s" % self	
		self._C1.name = "ohm's law of %s" % self
	
	def get_resistance(self):
		return self._resistance
	
	def set_resistance(self, r):
		self._resistance = r
		self._update_constraints()
	
	resistance = property(get_resistance, set_resistance)
	
	def __repr__(self):
		return "<%s (%f ohms) at %s>" % \
			(self.__class__.__name__, self._resistance, self.name or hex(id(self)))