from __future__ import absolute_import
from circuitsim.component import Component
from solver4 import Expr, Variable, Constraint

class SimpleSwitch(Component):
	
	def __init__(self, circuit=None):
		Component.__init__(self)
		
		self._state = "Open"
		
		self._current = Variable()
		self._current.value = 0.0
		self._vars.add(self._current)
		
		self._C1 = Constraint(Expr.constant(1))
		self._C1.max_iterations = 1
		self._constraints.add(self._C1)
		
		self._update_names()
		
		self._pins[0] = None
		self._pins[1] = None
		
		if circuit:
			self.install(circuit)
	
	def current(self, pin):
		if pin==0:
			return self._current.value
		else:
			return -self._current.value
	
	def _get_current(self, pin):
		if pin==0:
			return Expr.variable(self._current)
		else:
			return Expr.product(Expr.variable(self._current), Expr.constant(-1))
	
	def _update_constraints(self):
		if self._state == "Open":
			# I = 0
			self._C1.expr = Expr.variable(self._current)
		else:
			# V[0] = V[1]
			self._C1.expr = Expr.sum(
				Expr.variable(self._pins[0]._voltage),
				Expr.product(Expr.constant(-1), Expr.variable(self._pins[1]._voltage))
				)
	
	def _update_names(self):
		self._current.name = "current of %s" % self	
		self._C1.name = "switch of %s" % self
	
	def open(self):
		self._state = "Open"
		self._update_constraints()
	
	def close(self):
		self._state = "Closed"
		self._update_constraints()
	
	def set_state(self, state):
		assert state in ("Open", "Closed")
		if state=="Open":
			self.open()
		else:
			self.close()
	
	def get_state(self):
		return self._state
	
	state = property(get_state, set_state)
	
	def __repr__(self):
		return "<%s (%s) at %s>" % \
			(self.__class__.__name__, self._state, self.name or hex(id(self)))