from __future__ import absolute_import
from circuitsim.component import Component
from solver4 import Expr, Variable, Constraint

class VoltageSource(Component):
	""" Pin #0 is the negative pin. """
	
	def __init__(self, voltage, circuit=None):
		Component.__init__(self)
		
		self._voltage = voltage
		
		self._current = Variable()
		self._current.value = 0.0
		self._vars.add(self._current)
		
		self._C1 = Constraint(Expr.constant(1))
		self._C1.max_iterations = 1
		self._constraints.add(self._C1)
		
		self._update_names()
		
		self._pins[0] = None
		self._pins[1] = None
				
		if circuit:
			self.install(circuit)
	
	def current(self, pin):
		if pin==0:
			return self._current.value
		else:
			return -self._current.value
	
	def _get_current(self, pin):
		if pin==0:
			return Expr.variable(self._current)
		else:
			return Expr.product(Expr.constant(-1), Expr.variable(self._current))
	
	def _update_constraints(self):
		# V[0] - V[1] + V = 0
		self._C1.expr = Expr.sum(
			Expr.variable(self._pins[0]._voltage),
			Expr.product(Expr.constant(-1), Expr.variable(self._pins[1]._voltage)),
			Expr.constant(self._voltage)
			)
	
	def _update_names(self):
		self._current.name = "current of %s" % self
		self._C1.name = "voltage constraint of %s" % self
	
	def get_voltage(self):
		return self._voltage
	
	def set_voltage(self, c):
		self._voltage = c
		self._update_constraints()
	
	voltage = property(get_voltage, set_voltage)
	
	def __repr__(self):
		return "<%s (%f V) at %s>" % \
			(self.__class__.__name__, self._voltage, self.name or hex(id(self)))
		
		